@@ -300,6 +300,8 @@ class XEP_0030(BasePlugin):
 
     async def get_info_from_domain(self, domain=None, timeout=None,
                                    cached=True, callback=None):
+        """Fetch disco#info of specified domain and one disco#items level below"""
+
         if domain is None:
             domain = self.xmpp.boundjid.domain
 
@@ -348,7 +350,7 @@ class XEP_0030(BasePlugin):
                         combination handled by this Slixmpp instance and
                         no stanzas need to be sent.
                         Otherwise, a disco stanza must be sent to the
-                        remove JID to retrieve the info.
+                        remote JID to retrieve the info.
             cached   -- If true, then look for the disco info data from
                         the local cache system. If no results are found,
                         send the query as usual. The self.use_cache
