@@ -74,7 +74,7 @@ class XEP_0050(BasePlugin):
 
     name = 'xep_0050'
     description = 'XEP-0050: Ad-Hoc Commands'
-    dependencies = set(['xep_0030', 'xep_0004'])
+    dependencies = {'xep_0030', 'xep_0004'}
     stanza = stanza
     default_config = {
         'session_db': None
@@ -89,31 +89,17 @@ class XEP_0050(BasePlugin):
         self.commands = {}
 
         self.xmpp.register_handler(
-                Callback("Ad-Hoc Execute",
-                         StanzaPath('iq@type=set/command'),
-                         self._handle_command))
+            Callback("Ad-Hoc Execute",
+                     StanzaPath('iq@type=set/command'),
+                     self._handle_command))
 
         register_stanza_plugin(Iq, Command)
         register_stanza_plugin(Command, Form, iterable=True)
 
-        self.xmpp.add_event_handler('command_execute',
-                                    self._handle_command_start)
-        self.xmpp.add_event_handler('command_next',
-                                    self._handle_command_next)
-        self.xmpp.add_event_handler('command_cancel',
-                                    self._handle_command_cancel)
-        self.xmpp.add_event_handler('command_complete',
-                                    self._handle_command_complete)
+        self.xmpp.add_event_handler('command', self._handle_command_all)
 
     def plugin_end(self):
-        self.xmpp.del_event_handler('command_execute',
-                                    self._handle_command_start)
-        self.xmpp.del_event_handler('command_next',
-                                    self._handle_command_next)
-        self.xmpp.del_event_handler('command_cancel',
-                                    self._handle_command_cancel)
-        self.xmpp.del_event_handler('command_complete',
-                                    self._handle_command_complete)
+        self.xmpp.del_event_handler('command', self._handle_command_all)
         self.xmpp.remove_handler('Ad-Hoc Execute')
         self.xmpp['xep_0030'].del_feature(feature=Command.namespace)
         self.xmpp['xep_0030'].set_items(node=Command.namespace, items=tuple())
@@ -201,8 +187,27 @@ class XEP_0050(BasePlugin):
 
     def _handle_command(self, iq):
         """Raise command events based on the command action."""
+        self.xmpp.event('command', iq)
         self.xmpp.event('command_%s' % iq['command']['action'], iq)
 
+    def _handle_command_all(self, iq: Iq) -> None:
+        action = iq['command']['action']
+        sessionid = iq['command']['sessionid']
+        session = self.sessions.get(sessionid)
+
+        if session is None:
+            return self._handle_command_start(iq)
+
+        if action in ('next', 'execute'):
+            return self._handle_command_next(iq)
+        if action == 'prev':
+            return self._handle_command_prev(iq)
+        if action == 'complete':
+            return self._handle_command_complete(iq)
+        if action == 'cancel':
+            return self._handle_command_cancel(iq)
+        return None
+
     def _handle_command_start(self, iq):
         """
         Process an initial request to execute a command.
@@ -225,8 +230,8 @@ class XEP_0050(BasePlugin):
         if len(payload) == 1:
             payload = payload[0]
 
-        interfaces = set([item.plugin_attrib for item in payload])
-        payload_classes = set([item.__class__ for item in payload])
+        interfaces = {item.plugin_attrib for item in payload}
+        payload_classes = {item.__class__ for item in payload}
 
         initial_session = {'id': sessionid,
                            'from': iq['from'],
@@ -322,8 +327,8 @@ class XEP_0050(BasePlugin):
         interfaces = session.get('interfaces', set())
         payload_classes = session.get('payload_classes', set())
 
-        interfaces.update(set([item.plugin_attrib for item in payload]))
-        payload_classes.update(set([item.__class__ for item in payload]))
+        interfaces.update({item.plugin_attrib for item in payload})
+        payload_classes.update({item.__class__ for item in payload})
 
         session['interfaces'] = interfaces
         session['payload_classes'] = payload_classes
@@ -468,7 +473,7 @@ class XEP_0050(BasePlugin):
                                                **kwargs)
 
     def send_command(self, jid, node, ifrom=None, action='execute',
-                    payload=None, sessionid=None, flow=False, **kwargs):
+                     payload=None, sessionid=None, flow=False, **kwargs):
         """
         Create and send a command stanza, without using the provided
         workflow management APIs.
@@ -611,7 +616,7 @@ class XEP_0050(BasePlugin):
     def terminate_command(self, session):
         """
         Delete a command's session after a command has completed
-        or an error has occured.
+        or an error has occurred.
 
         Arguments:
             session -- All stored data relevant to the current
