@@ -40,7 +40,7 @@ class XEP_0202(BasePlugin):
     }
 
     def plugin_init(self):
-        """Start the XEP-0203 plugin."""
+        """Start the XEP-0202 plugin."""
 
         if not self.local_time:
             def default_local_time(jid):
@@ -50,7 +50,7 @@ class XEP_0202(BasePlugin):
 
         self.xmpp.register_handler(
             Callback('Entity Time',
-                 StanzaPath('iq/entity_time'),
+                 StanzaPath('iq@type=get/entity_time'),
                  self._handle_time_request))
         register_stanza_plugin(Iq, stanza.EntityTime)
 
