@@ -19,7 +19,7 @@ class XEP_0279(BasePlugin):
 
     name = 'xep_0279'
     description = 'XEP-0279: Server IP Check'
-    dependencies = set(['xep_0030'])
+    dependencies = {'xep_0030'}
     stanza = stanza
 
     def plugin_init(self):
@@ -31,11 +31,11 @@ class XEP_0279(BasePlugin):
     def plugin_end(self):
         self.xmpp['xep_0030'].del_feature(feature='urn:xmpp:sic:0')
 
-    def check_ip(self, ifrom=None, block=True, timeout=None, callback=None,
+    def check_ip(self, ifrom=None, timeout=None, callback=None,
                       timeout_callback=None):
         iq = self.xmpp.Iq()
         iq['type'] = 'get'
         iq['from'] = ifrom
         iq.enable('ip_check')
-        return iq.send(block=block, timeout=timeout, callback=callback,
+        return iq.send(timeout=timeout, callback=callback,
                        timeout_callback=timeout_callback)
