@@ -6,7 +6,6 @@
     See the file LICENSE for copying permission.
 """
 
-import asyncio
 import logging
 import os.path
 
@@ -29,7 +28,13 @@ class UploadServiceNotFound(FileUploadError):
     pass
 
 class FileTooBig(FileUploadError):
-    pass
+    def __str__(self):
+        return 'File size too large: {} (max: {} bytes)' \
+            .format(self.args[0], self.args[1])
+
+class HTTPError(FileUploadError):
+    def __str__(self):
+        return 'Could not upload file: %d (%s)' % (self.args[0], self.args[1])
 
 class XEP_0363(BasePlugin):
     ''' This plugin only supports Python 3.5+ '''
@@ -68,12 +73,18 @@ class XEP_0363(BasePlugin):
     def _handle_request(self, iq):
         self.xmpp.event('http_upload_request', iq)
 
-    async def find_upload_service(self, timeout=None):
-        results = await self.xmpp['xep_0030'].get_info_from_domain()
+    async def find_upload_service(self, domain=None, timeout=None):
+        results = await self.xmpp['xep_0030'].get_info_from_domain(
+            domain=domain, timeout=timeout)
 
+        candidates = []
         for info in results:
             for identity in info['disco_info']['identities']:
                 if identity[0] == 'store' and identity[1] == 'file':
+                    candidates.append(info)
+        for info in candidates:
+            for feature in info['disco_info']['features']:
+                if feature == Request.namespace:
                     return info
 
     def request_slot(self, jid, filename, size, content_type=None, ifrom=None,
@@ -90,11 +101,12 @@ class XEP_0363(BasePlugin):
                        timeout_callback=timeout_callback)
 
     async def upload_file(self, filename, size=None, content_type=None, *,
-                          input_file=None, ifrom=None, timeout=None,
+                          input_file=None, ifrom=None, domain=None, timeout=None,
                           callback=None, timeout_callback=None):
         ''' Helper function which does all of the uploading process. '''
         if self.upload_service is None:
-            info_iq = await self.find_upload_service(timeout=timeout)
+            info_iq = await self.find_upload_service(
+                domain=domain, timeout=timeout)
             if info_iq is None:
                 raise UploadServiceNotFound()
             self.upload_service = info_iq['from']
@@ -106,7 +118,7 @@ class XEP_0363(BasePlugin):
                     except (TypeError, ValueError):
                         log.error('Invalid max size received from HTTP File Upload service')
                         self.max_file_size = float('+inf')
-                break
+                    break
 
         if input_file is None:
             input_file = open(filename, 'rb')
@@ -116,7 +128,7 @@ class XEP_0363(BasePlugin):
             input_file.seek(0)
 
         if size > self.max_file_size:
-            raise FileTooBig()
+            raise FileTooBig(size, self.max_file_size)
 
         if content_type is None:
             content_type = guess_type(filename)[0]
@@ -125,7 +137,8 @@ class XEP_0363(BasePlugin):
 
         basename = os.path.basename(filename)
         slot_iq = await self.request_slot(self.upload_service, basename, size,
-                                               content_type, ifrom, timeout)
+                                          content_type, ifrom, timeout,
+                                          timeout_callback=timeout_callback)
         slot = slot_iq['http_upload_slot']
 
         headers = {
@@ -141,6 +154,8 @@ class XEP_0363(BasePlugin):
                     data=input_file,
                     headers=headers,
                     timeout=timeout)
+            if response.status >= 400:
+                raise HTTPError(response.status, await response.text())
             log.info('Response code: %d (%s)', response.status, await response.text())
             response.close()
             return slot['get']['url']
